@@ -25,12 +25,8 @@
 #endif
 #include <QSet>
 #include <QTemporaryDir>
-#include <QElapsedTimer>
-#include <QApplication>
 #include <QMap>
 #include <QSet>
-#include <QFileSystemWatcher>
-#include <QNetworkAccessManager>
 #include <QSortFilterProxyModel>
 #include <QToolButton>
 #include <QProgressBar>
@@ -44,6 +40,12 @@
 #include "htmlwindow.h"
 #include "moocsubmission.h"
 #include "solverconfiguration.h"
+#include "process.h"
+#include "codechecker.h"
+
+#include "../cp-profiler/src/cpprofiler/conductor.hh"
+#include "../cp-profiler/src/cpprofiler/execution_window.hh"
+#include "../cp-profiler/src/cpprofiler/analysis/merge_window.hh"
 
 namespace Ui {
 class MainWindow;
@@ -54,85 +56,6 @@ class MainWindow;
 class QNetworkReply;
 class QTextStream;
 
-class IDEStatistics {
-public:
-    int errorsShown;
-    int errorsClicked;
-    int modelsRun;
-    QStringList solvers;
-    QVariantMap toVariantMap(void);
-    IDEStatistics(void);
-    void init(QVariant v);
-    QByteArray toJson(void);
-    void resetCounts(void);
-};
-
-class IDE : public QApplication {
-    Q_OBJECT
-public:
-    IDE(int& argc, char* argv[]);
-    ~IDE(void);
-    struct Doc;
-    typedef QMap<QString,Doc*> DMap;
-    DMap documents;
-    typedef QMap<QString,MainWindow*> PMap;
-    PMap projects;
-    QSet<MainWindow*> mainWindows;
-
-    QStringList recentFiles;
-    QStringList recentProjects;
-
-    QSet<QString> modifiedFiles;
-    QTimer modifiedTimer;
-
-    IDEStatistics stats;
-
-    MainWindow* lastDefaultProject;
-    QMainWindow* cheatSheet;
-
-    QNetworkAccessManager* networkManager;
-    QNetworkReply* versionCheckReply;
-
-#ifdef Q_OS_MAC
-    QMenuBar* defaultMenuBar;
-    QMenu* recentFilesMenu;
-    QMenu* recentProjectsMenu;
-public slots:
-    void recentFileMenuAction(QAction*);
-    void recentProjectMenuAction(QAction*);
-public:
-#endif
-
-    QFileSystemWatcher fsWatch;
-
-    bool hasFile(const QString& path);
-    QPair<QTextDocument*,bool> loadFile(const QString& path, QWidget* parent);
-    void loadLargeFile(const QString& path, QWidget* parent);
-    QTextDocument* addDocument(const QString& path, QTextDocument* doc, CodeEditor* ce);
-    void registerEditor(const QString& path, CodeEditor* ce);
-    void removeEditor(const QString& path, CodeEditor* ce);
-    void renameFile(const QString& oldPath, const QString& newPath);
-    QString appDir(void) const;
-    static IDE* instance(void);
-    QString getLastPath(void);
-    void setLastPath(const QString& path);
-    void setEditorFont(QFont font);
-    void addRecentFile(const QString& file);
-    void addRecentProject(const QString& file);
-protected:
-    bool event(QEvent *);
-protected slots:
-    void versionCheckFinished(void);
-    void newProject(void);
-    void openFile(const QString& filename = QString(""));
-    void fileModified(const QString&);
-    void fileModifiedTimeout(void);
-    void handleFocusChange(QWidget*,QWidget*);
-public slots:
-    void checkUpdate(void);
-    void help(void);
-};
-
 class MainWindow : public QMainWindow
 {
     Q_OBJECT
@@ -143,8 +66,19 @@ public:
     ~MainWindow();
 
 private:
+    enum CompileMode { CM_RUN, CM_COMPILE, CM_PROFILE };
+
     void init(const QString& project);
-    void compileOrRun();
+    void compileOrRun(
+            CompileMode cm,
+            const SolverConfiguration* sc = nullptr,
+            const QString& model = QString(),
+            const QStringList& data = QStringList(),
+            const QString& checker = QString(),
+            const QStringList& extraArgs = QStringList());
+    bool getModelParameters(const SolverConfiguration& sc, const QString& model, QStringList& data, QStringList& extraArgs);
+    QString currentModelFile(void);
+    bool promptSaveModified(void);
     void setEditorMenuItemsEnabled(bool enabled);
 signals:
     /// emitted when compilation and running of a model has finished
@@ -153,7 +87,7 @@ signals:
 public slots:
 
     void openFile(const QString &path = QString(), bool openAsModified=false, bool focus=true);
-    void on_actionStop_triggered();
+    void stop();
 
 private slots:
 
@@ -167,37 +101,19 @@ private slots:
 
     void on_actionRun_triggered();
 
-    void checkArgs(QString filepath);
-    void checkArgsOutput();
-    void checkArgsFinished(int exitcode, QProcess::ExitStatus exitstatus);
-
-    void checkModelStarted();
-    void checkModelFinished(int exitcode, QProcess::ExitStatus exitstatus);
-    void checkModelError(QProcess::ProcessError);
-
-    void readOutput();
-    void readRrofileOutput();
-
-    void procFinished(int, bool showTime=true);
-
-    void outputProcFinished(int, bool showTime=true);
-
-    void procError(QProcess::ProcessError);
-    void checkArgsError(QProcess::ProcessError);
-    void outputProcError(QProcess::ProcessError);
+    void procFinished(int, qint64 time);
+    void procFinished(int);
 
     void on_actionSave_triggered();
     void on_actionQuit_triggered();
 
-    void statusTimerEvent();
+    void statusTimerEvent(qint64 time);
 
     void on_actionCompile_triggered();
 
-    void openCompiledFzn(int);
+    void openCompiledFzn(const QString& file);
 
-    void profileCompiledFzn(int);
-
-    void runCompiledFzn(int,QProcess::ExitStatus);
+    void profileCompiledFzn(const QString& output);
 
     void on_actionSave_as_triggered();
 
@@ -243,6 +159,10 @@ private slots:
 
     void on_actionSave_all_triggered();
 
+    void on_actionSave_solver_configuration_triggered();
+
+    void on_actionSave_all_solver_configurations_triggered();
+
     void on_action_Un_comment_triggered();
 
     void on_actionOnly_editor_triggered();
@@ -263,20 +183,6 @@ private slots:
 
     void on_actionShow_project_explorer_triggered();
 
-    void activateFileInProject(const QModelIndex&);
-
-    void onProjectCustomContextMenu(const QPoint&);
-
-    void onActionProjectOpen_triggered();
-
-    void onActionProjectRemove_triggered();
-
-    void onActionProjectRename_triggered();
-
-    void onActionProjectRunWith_triggered();
-
-    void onActionProjectAdd_triggered();
-
     void on_actionNewModel_file_triggered();
 
     void on_actionNewData_file_triggered();
@@ -285,8 +191,6 @@ private slots:
 
     void fileRenamed(const QString&, const QString&);
 
-    void on_conf_timeLimit_valueChanged(int arg1);
-
     void onClipboardChanged();
 
     void editor_cursor_position_changed();
@@ -299,28 +203,15 @@ private slots:
 
     void on_actionDark_mode_toggled(bool arg1);
 
+    //Themes
+    void on_actionMangoTheme_triggered();
+    void on_actionBlueberryTheme_triggered();
+    void on_actionDefaultTheme_triggered();
+
     void moocFinished(int);
 
-    void on_defaultBehaviourButton_toggled(bool checked);
-
     void on_actionEditSolverConfig_triggered();
 
-    void on_conf_solver_conf_currentIndexChanged(int index);
-
-    void on_solverConfigurationSelected(QAction*);
-
-    void on_cloneSolverConfButton_clicked();
-    void on_deleteSolverConfButton_clicked();
-    void on_renameSolverConfButton_clicked();
-
-    void on_solverConfNameEdit_returnPressed();
-    void on_solverConfNameEdit_escPressed();
-
-    void on_conf_default_toggled(bool checked);
-    void on_conf_dock_widget_visibilityChanged(bool visible);
-
-    void on_conf_check_solutions_toggled(bool checked);
-
     void on_b_next_clicked();
 
     void on_b_prev_clicked();
@@ -335,10 +226,38 @@ private slots:
 
     void on_find_textEdited(const QString &arg1);
 
-    void on_extraOptionsBox_toggled(bool arg1);
-
     void on_actionProfile_compilation_triggered();
 
+    void on_configWindow_dockWidget_visibilityChanged(bool visible);
+
+    void on_config_window_itemsChanged(const QStringList& items);
+
+    void on_config_window_selectedIndexChanged(int );
+
+    void on_moocChanged(const MOOCAssignment* mooc);
+
+    void on_projectBrowser_runRequested(const QStringList& files);
+
+    void on_projectBrowser_openRequested(const QStringList& files);
+
+    void on_projectBrowser_removeRequested(const QStringList& files);
+
+    void on_actionShow_search_profiler_triggered();
+
+    void on_actionProfile_search_triggered();
+
+    void showExecutionWindow(cpprofiler::ExecutionWindow& e);
+    void showMergeWindow(cpprofiler::analysis::MergeWindow& m);
+
+    void on_cpprofiler_dockWidget_visibilityChanged(bool visible);
+
+    void on_menuSolver_configurations_triggered(QAction* action);
+
+    void on_solutionOutput(const QString& solution);
+    void on_finalStatus(const QString& status);
+    void on_fragment(const QString& data);
+    void on_progressOutput(float progress);
+
 protected:
     virtual void closeEvent(QCloseEvent*);
     virtual void dragEnterEvent(QDragEnterEvent *);
@@ -347,19 +266,18 @@ protected:
     virtual void paintEvent(QPaintEvent *);
 #endif
     bool eventFilter(QObject *, QEvent *);
-    void openJSONViewer(void);
-    void finishJSONViewer(void);
-
-    SolverConfiguration* getCurrentSolverConfig(void);
-    Solver* getCurrentSolver(void);
-
-    void compileAndRun(const QString& modelPath, const QString& additionalCmdlineParams, const QStringList& additionalDataFiles);
+    void compileAndRun(const QString& modelPath, const QString& additionalCmdlineParams, const QStringList& additionalDataFiles, const QStringList& additionalMznParams);
 public:
-    bool runWithOutput(const QString& modelFile, const QString& dataFile, int timeout, QTextStream& outstream);
     void resolve(int htmlWindowIdentifier, const QString &data);
     QString currentSolver(void) const;
     QString currentSolverConfigName(void);
     int addHtmlWindow(HTMLWindow* w);
+    bool checkSolutions(void) const;
+    void setCheckSolutions(bool b);
+    SolverConfiguration* getCurrentSolverConfig(void);
+    const Solver* getCurrentSolver(void);
+    void compile(const SolverConfiguration& sc, const QString& model, const QStringList& data = QStringList(), const QStringList& extraArgs = QStringList(), bool profile = false);
+    void run(const SolverConfiguration& sc, const QString& model, const QStringList& data = QStringList(), const QStringList& extraArgs = QStringList(), QTextStream* ts = nullptr);
 private:
     Ui::MainWindow *ui;
     CodeEditor* curEditor;
@@ -367,71 +285,28 @@ private:
     int curHtmlWindow;
     QVector<HTMLWindow*> htmlWindows;
     QVector<QString> htmlWindowModels;
-    MznProcess* process;
-    MznProcess* check_process;
-    QString processName;
-    QString curModelFilepath;
+    CodeChecker* code_checker;
     CodeEditor* curCheckEditor;
-    MznProcess* outputProcess;
-    bool processWasStopped;
     int solutionCount;
     int solutionLimit;
     QTimer solutionLimitTimer;
     QVector<QString> hiddenSolutions;
-    int curJSONHandler;
-    bool inJSONHandler;
-    bool isJSONinitHandler;
-    QStringList htmlBuffer;
-    bool inHTMLHandler {false};
-    bool hadNonJSONOutput;
-    QVector<QStringList> JSONOutput;
-    QTimer* timer;
-    QTimer* solverTimeout;
-    QTimer* checkTimer;
-    int time;
-    QElapsedTimer elapsedTime;
     QProgressBar* progressBar;
     QLabel* statusLabel;
     QLabel* statusLineColLabel;
     QFont editorFont;
     bool darkMode;
     QVector<Solver> solvers;
-    QString userSolverConfigDir;
-    QString userConfigFile;
-    QString mznStdlibDir;
-    int defaultSolverIdx;
-    QString mznDistribPath;
-    QString getMznDistribPath(void) const;
-    QString currentFznTarget;
-    QString currentPathsTarget;
-    bool isOptimisation;
     QStringList currentAdditionalDataFiles;
-    bool runSolns2Out;
     QTemporaryDir* tmpDir;
     QVector<QTemporaryDir*> cleanupTmpDirs;
-    QVector<MznProcess*> cleanupProcesses;
+    QVector<Process*> cleanupProcesses;
     QTextCursor incrementalFindCursor;
-    QString projectPath;
     bool saveBeforeRunning;
-    QString compileErrors;
-    QString checkArgsStdout;
     ParamDialog* paramDialog;
-    enum CompileMode { CM_RUN, CM_COMPILE, CM_PROFILE };
-    CompileMode compileMode;
     bool profileInfoVisible;
     int runTimeout;
-    QString minizinc_executable;
-    QVersionNumber minizinc_version;
-    Project project;
-    QSortFilterProxyModel* projectSort;
-    QMenu* projectContextMenu;
-    QAction* projectOpen;
-    QAction* projectRemove;
-    QAction* projectRename;
-    QAction* projectRunWith;
-    QAction* projectAdd;
-    QString projectSelectedFile;
-    QModelIndex projectSelectedIndex;
+    Project* project;
     int newFileCounter;
     QComboBox* solverConfCombo;
     QAction* fakeRunAction;
@@ -443,38 +318,46 @@ private:
     bool processRunning;
 
     QToolButton* runButton;
-    QVector<SolverConfiguration> projectSolverConfigs;
-    QVector<SolverConfiguration> builtinSolverConfigs;
     QVector<QPair<SolverFlag,QWidget*>> extraSolverFlags;
     int currentSolverConfig;
     bool renamingSolverConf;
 
+    cpprofiler::Conductor* conductor = nullptr;
+    int ex_id;
+
     void createEditor(const QString& path, bool openAsModified, bool isNewFile, bool readOnly=false, bool focus=true);
     enum ConfMode { CONF_CHECKARGS, CONF_COMPILE, CONF_RUN };
-    QStringList parseConf(const ConfMode& confMode, const QString& modelFile, bool isOptimisation);
+    QStringList parseConf(const ConfMode& confMode, const QString& modelFile);
     void saveFile(CodeEditor* ce, const QString& filepath);
     void saveProject(const QString& filepath);
     void loadProject(const QString& filepath);
     void setEditorFont(QFont font);
     void setLastPath(const QString& s);
     QString getLastPath(void);
-    QString setElapsedTime();
-    void checkMznPath();
+    QString setElapsedTime(qint64 elapsed_t);
+    void checkDriver();
     void updateRecentProjects(const QString& p);
     void updateRecentFiles(const QString& p);
-    void addFileToProject(bool dznOnly);
     void updateUiProcessRunning(bool pr);
     void highlightPath(QString& path, int index);
     QVector<CodeEditor*> collectCodeEditors(QVector<QStringList>& locs);
-    void updateSolverConfigs(void);
-    void setCurrentSolverConfig(int idx);
     void find(bool fwd, bool forceNoWrapAround=false);
+    bool requireMiniZinc(void);
+    void outputStdErr(const QString& line);
+    QStringList getOpenFiles(void);
+
+    void openJSONViewer(bool isJSONinitHandler, const QVector<SolveProcess::VisOutput>& output);
+    void finishJSONViewer(qint64 time);
+
+    void updateProfileSearchButton(void);
+
+    QString runMessage(const QString& action, const QString& model, const QStringList& data, const QStringList& extraArgs);
 public:
     void addOutput(const QString& s, bool html=true);
     void openProject(const QString& fileName);
     bool isEmptyProject(void);
     void selectJSONSolution(HTMLPage* source, int n);
-    const Project& getProject(void) { return project; }
+    Project& getProject(void) { return *project; }
 };
 
 #endif // MAINWINDOW_H
