@@ -12,7 +12,10 @@
 
 #include "solverdialog.h"
 #include "ui_solverdialog.h"
+#include "ide.h"
 #include "mainwindow.h"
+#include "process.h"
+#include "exception.h"
 #include <QDebug>
 #include <QMessageBox>
 #include <QFileDialog>
@@ -24,13 +27,6 @@
 #include <QGridLayout>
 #include <QLineEdit>
 
-#ifndef Q_OS_WIN
-#include <sys/types.h>
-#include <unistd.h>
-#include <signal.h>
-#include <iostream>
-#endif
-
 #include <QtGlobal>
 #ifdef Q_OS_WIN
 #define pathSep ";"
@@ -47,19 +43,192 @@
 #endif
 #endif
 
-SolverDialog::SolverDialog(QVector<Solver>& solvers0,
-                           QString& userSolverConfigDir0,
-                           QString& userConfigFile0,
-                           QString& mznStdlibDir0,
-                           bool openAsAddNew,
-                           const QString& mznPath, QWidget *parent) :
+Solver::Solver(const QJsonObject& sj) {
+    json = sj;
+    name = sj["name"].toString();
+    QJsonObject extraInfo = sj["extraInfo"].toObject();
+    executable_resolved = extraInfo["executable"].toString("");
+    mznlib_resolved = extraInfo["mznlib"].toString("");
+    configFile = extraInfo["configFile"].toString("");
+    if (extraInfo["defaultFlags"].isArray()) {
+        QJsonArray ei = extraInfo["defaultFlags"].toArray();
+        for (auto df : ei) {
+            defaultFlags.push_back(df.toString());
+        }
+    }
+    isDefaultSolver = (extraInfo["isDefault"].isBool() && extraInfo["isDefault"].toBool());
+    contact = sj["contact"].toString("");
+    website = sj["website"].toString("");
+    supportsFzn = sj["supportsFzn"].toBool(true);
+    supportsMzn = sj["supportsMzn"].toBool(false);
+    if (sj["requiredFlags"].isArray()) {
+        QJsonArray rfs = sj["requiredFlags"].toArray();
+        for (auto rf : rfs) {
+            requiredFlags.push_back(rf.toString());
+        }
+    }
+    if (sj["stdFlags"].isArray()) {
+        QJsonArray sfs = sj["stdFlags"].toArray();
+        for (auto sf : sfs) {
+            stdFlags.push_back(sf.toString());
+        }
+    }
+    if (sj["extraFlags"].isArray()) {
+        QJsonArray sfs = sj["extraFlags"].toArray();
+        for (auto sf : sfs) {
+            if (sf.isArray()) {
+                QJsonArray extraFlagA = sf.toArray();
+                if (extraFlagA.size()==4) {
+                    SolverFlag extraFlag;
+                    extraFlag.min=1.0;
+                    extraFlag.max=0.0;
+                    extraFlag.name = extraFlagA[0].toString();
+                    QRegularExpression re_opt("^(int|float|bool)(:([0-9a-zA-Z.-]+):([0-9a-zA-Z.-]+))?");
+                    QRegularExpressionMatch re_opt_match = re_opt.match(extraFlagA[2].toString());
+                    if (re_opt_match.hasMatch()) {
+                        if (re_opt_match.captured(1)=="int") {
+                            if (re_opt_match.captured(3).isEmpty()) {
+                                extraFlag.t = SolverFlag::T_INT;
+                            } else {
+                                extraFlag.t = SolverFlag::T_INT_RANGE;
+                                extraFlag.min_ll = re_opt_match.captured(3).toLongLong();
+                                extraFlag.max_ll = re_opt_match.captured(4).toLongLong();
+                            }
+                        } else if (re_opt_match.captured(1)=="float") {
+                            if (re_opt_match.captured(3).isEmpty()) {
+                                extraFlag.t = SolverFlag::T_FLOAT;
+                            } else {
+                                extraFlag.t = SolverFlag::T_FLOAT_RANGE;
+                                extraFlag.min = re_opt_match.captured(3).toDouble();
+                                extraFlag.max = re_opt_match.captured(4).toDouble();
+                            }
+                        } else if (re_opt_match.captured(1)=="bool") {
+                            if (re_opt_match.captured(3).isEmpty()) {
+                                extraFlag.t = SolverFlag::T_BOOL;
+                            } else {
+                                extraFlag.t = SolverFlag::T_BOOL_ONOFF;
+                                extraFlag.options = QStringList({re_opt_match.captured(3),re_opt_match.captured(4)});
+                            }
+                        }
+                    } else {
+                        if (extraFlagA[2].toString()=="string") {
+                            extraFlag.t = SolverFlag::T_STRING;
+                        } else if (extraFlagA[2].toString().startsWith("opt:")) {
+                            extraFlag.t = SolverFlag::T_OPT;
+                            extraFlag.options = extraFlagA[2].toString().mid(4).split(":");
+//                                        } else if (extraFlagA[2].toString()=="solver") {
+//                                            extraFlag.t = SolverFlag::T_SOLVER;
+                        } else {
+                            continue;
+                        }
+                    }
+                    extraFlag.description = extraFlagA[1].toString();
+                    auto def = extraFlagA[3].toString();
+                    switch (extraFlag.t) {
+                    case SolverFlag::T_INT:
+                    case SolverFlag::T_INT_RANGE:
+                        extraFlag.def = def.toLongLong();
+                        break;
+                    case SolverFlag::T_BOOL:
+                        extraFlag.def = def == "true";
+                        break;
+                    case SolverFlag::T_BOOL_ONOFF:
+                        if (def == extraFlag.options[0]) {
+                            extraFlag.def = true;
+                        } else if (def == extraFlag.options[1]) {
+                            extraFlag.def = false;
+                        } else {
+                            extraFlag.def = def == "true";
+                        }
+                        break;
+                    case SolverFlag::T_FLOAT:
+                    case SolverFlag::T_FLOAT_RANGE:
+                        extraFlag.def = def.toDouble();
+                        break;
+                    case SolverFlag::T_STRING:
+                    case SolverFlag::T_OPT:
+                    case SolverFlag::T_SOLVER:
+                        extraFlag.def = def;
+                        break;
+                    }
+                    extraFlags.push_back(extraFlag);
+                }
+            }
+        }
+    }
+    isGUIApplication = sj["isGUIApplication"].toBool(false);
+    needsMznExecutable = sj["needsMznExecutable"].toBool(false);
+    needsStdlibDir = sj["needsStdlibDir"].toBool(false);
+    needsPathsFile = sj["needsPathsFile"].toBool(false);
+    needsSolns2Out = sj["needsSolns2Out"].toBool(true);
+    executable = sj["executable"].toString("");
+    id = sj["id"].toString();
+    version = sj["version"].toString();
+    mznlib = sj["mznlib"].toString();
+    mznLibVersion = sj["mznlibVersion"].toInt();
+}
+
+Solver& Solver::lookup(const QString& str)
+{
+    MznProcess p;
+    try {
+        auto result = p.run({ "--solver-json", str });
+        auto solver_doc = QJsonDocument::fromJson(result.stdOut.toUtf8());
+        if (!solver_doc.isObject()) {
+            throw ConfigError("Failed to find solver " + str);
+        }
+        auto solver = Solver(solver_doc.object());
+        for (auto& s: MznDriver::get().solvers()) {
+            if (solver == s) {
+                return s;
+            }
+        }
+        MznDriver::get().solvers() << solver;
+        return MznDriver::get().solvers().last();
+    } catch (Exception&) {
+        throw DriverError("Failed to lookup solver " + str);
+    }
+}
+
+Solver& Solver::lookup(const QString& id, const QString& version, bool strict)
+{
+    if (strict) {
+        return lookup(id + "@" + version);
+    }
+
+    try {
+        return lookup(id + "@" + version);
+    } catch (ConfigError&) {
+        return lookup(id);
+    }
+}
+
+bool Solver::operator==(const Solver& s) const {
+    if (configFile.isEmpty() && s.configFile.isEmpty()) {
+        return id == s.id && version == s.version;
+    }
+    return configFile == s.configFile;
+}
+
+bool Solver::hasAllRequiredFlags()
+{
+    for (auto& rf : requiredFlags) {
+        if (!defaultFlags.contains(rf)) {
+            return false;
+        }
+    }
+    return true;
+}
+
+SolverDialog::SolverDialog(bool openAsAddNew, QWidget *parent) :
     QDialog(parent),
-    ui(new Ui::SolverDialog),
-    solvers(solvers0), userSolverConfigDir(userSolverConfigDir0), userConfigFile(userConfigFile0), mznStdlibDir(mznStdlibDir0)
+    ui(new Ui::SolverDialog)
 {
     ui->setupUi(this);
 
-    ui->mznDistribPath->setText(mznPath);
+    auto& driver = MznDriver::get();
+    auto& solvers = driver.solvers();
+    ui->mznDistribPath->setText(driver.mznDistribPath());
     for (int i=solvers.size(); i--;) {
         ui->solvers_combo->insertItem(0,solvers[i].name+" "+solvers[i].version,i);
     }
@@ -67,6 +236,16 @@ SolverDialog::SolverDialog(QVector<Solver>& solvers0,
     QSettings settings;
     settings.beginGroup("ide");
     ui->check_updates->setChecked(settings.value("checkforupdates21",false).toBool());
+    ui->checkSolutions_checkBox->setChecked(settings.value("checkSolutions", true).toBool());
+    ui->clearOutput_checkBox->setChecked(settings.value("clearOutput", false).toBool());
+    int compressSolutions = settings.value("compressSolutions", 100).toInt();
+    if (compressSolutions > 0) {
+        ui->compressSolutions_spinBox->setValue(compressSolutions);
+        ui->compressSolutions_checkBox->setChecked(true);
+    }
+    else {
+        ui->compressSolutions_checkBox->setChecked(false);
+    }
     settings.endGroup();
     if (openAsAddNew)
         ui->solvers_combo->setCurrentIndex(ui->solvers_combo->count()-1);
@@ -79,11 +258,6 @@ SolverDialog::~SolverDialog()
     delete ui;
 }
 
-QString SolverDialog::mznPath()
-{
-    return ui->mznDistribPath->text();
-}
-
 namespace {
 
     void clearRequiredFlagsLayout(QGridLayout* l) {
@@ -109,9 +283,11 @@ namespace {
 
 void SolverDialog::on_solvers_combo_currentIndexChanged(int index)
 {
+    auto& driver = MznDriver::get();
+    auto& solvers = driver.solvers();
     QGridLayout* rfLayout = static_cast<QGridLayout*>(ui->requiredFlags->layout());
     clearRequiredFlagsLayout(rfLayout);
-    QString userSolverConfigCanonical = QFileInfo(userSolverConfigDir).canonicalPath();
+    QString userSolverConfigCanonical = QFileInfo(driver.userSolverConfigDir()).canonicalPath();
     if (index<solvers.size()) {
         ui->name->setText(solvers[index].name);
         ui->solverId->setText(solvers[index].id);
@@ -187,6 +363,11 @@ void SolverDialog::on_solvers_combo_currentIndexChanged(int index)
 
 void SolverDialog::on_updateButton_clicked()
 {
+    auto& driver = MznDriver::get();
+    auto& solvers = driver.solvers();
+    auto& userConfigFile = driver.userConfigFile();
+    auto& userSolverConfigDir = driver.userSolverConfigDir();
+
     if (!ui->requiredFlags->isHidden()) {
         // This is an internal solver, we are only updating the required flags
 
@@ -371,6 +552,8 @@ void SolverDialog::on_updateButton_clicked()
 
 void SolverDialog::on_deleteButton_clicked()
 {
+    auto& driver = MznDriver::get();
+    auto& solvers = driver.solvers();
     int index = ui->solvers_combo->currentIndex();
     if (QMessageBox::warning(this,"MiniZinc IDE","Delete solver "+solvers[index].name+"?",QMessageBox::Ok | QMessageBox::Cancel)
             == QMessageBox::Ok) {
@@ -415,194 +598,25 @@ void SolverDialog::on_check_updates_stateChanged(int checkstate)
     settings.endGroup();
 }
 
-void SolverDialog::checkMznExecutable(const QString& mznDistribPath,
-                                      QString& minizinc_executable,
-                                      QString& minizinc_version_string,
-                                      QVector<Solver>& solvers,
-                                      QString& userSolverConfigDir,
-                                      QString& userConfigFile,
-                                      QString& mznStdlibDir)
-{
-    MznProcess p;
-    QStringList args;
-    args << "--version";
-    minizinc_executable = "";
-    minizinc_version_string = "";
-    p.start("minizinc", args, mznDistribPath);
-    if (!p.waitForStarted() || !p.waitForFinished()) {
-        return;
-    } else {
-        minizinc_executable = "minizinc";
-    }
-    minizinc_version_string = p.readAllStandardOutput()+p.readAllStandardError();
-    QRegExp reVersion("version ([0-9]+)\\.([0-9]+)\\.\\S+");
-    if (reVersion.indexIn(minizinc_version_string)==-1) {
-        minizinc_executable = "";
-        return;
-    } else {
-        bool ok;
-        int curVersionMajor = reVersion.cap(1).toInt(&ok);
-        if (ok) {
-            int curVersionMinor = reVersion.cap(2).toInt(&ok);
-            if (curVersionMajor<2 || (curVersionMajor<3 && curVersionMinor<2)) {
-                minizinc_executable = "";
-                return;
-            }
-        }
-    }
-    if (!minizinc_executable.isEmpty()) {
-        args.clear();
-        args << "--config-dirs";
-        p.start(minizinc_executable, args, mznDistribPath);
-        if (p.waitForStarted() && p.waitForFinished()) {
-            QString allOutput = p.readAllStandardOutput();
-            QJsonDocument jd = QJsonDocument::fromJson(allOutput.toUtf8());
-            if (!jd.isNull()) {
-                QJsonObject sj = jd.object();
-                userSolverConfigDir = sj["userSolverConfigDir"].toString();
-                userConfigFile = sj["userConfigFile"].toString();
-                mznStdlibDir = sj["mznStdlibDir"].toString();
-            }
-        }
-        args.clear();
-        args << "--solvers-json";
-        p.start(minizinc_executable, args, mznDistribPath);
-        if (p.waitForStarted() && p.waitForFinished()) {
-            QString allOutput = p.readAllStandardOutput();
-            QJsonDocument jd = QJsonDocument::fromJson(allOutput.toUtf8());
-            if (!jd.isNull()) {
-                solvers.clear();
-                QJsonArray allSolvers = jd.array();
-                for (int i=0; i<allSolvers.size(); i++) {
-                    QJsonObject sj = allSolvers[i].toObject();
-                    Solver s;
-                    s.json = sj;
-                    s.name = sj["name"].toString();
-                    QJsonObject extraInfo = sj["extraInfo"].toObject();
-                    s.executable_resolved = extraInfo["executable"].toString("");
-                    s.mznlib_resolved = extraInfo["mznlib"].toString("");
-                    s.configFile = extraInfo["configFile"].toString("");
-                    if (extraInfo["defaultFlags"].isArray()) {
-                        QJsonArray ei = extraInfo["defaultFlags"].toArray();
-                        for (auto df : ei) {
-                            s.defaultFlags.push_back(df.toString());
-                        }
-                    }
-                    s.isDefaultSolver = (extraInfo["isDefault"].isBool() && extraInfo["isDefault"].toBool());
-                    s.contact = sj["contact"].toString("");
-                    s.website = sj["website"].toString("");
-                    s.supportsFzn = sj["supportsFzn"].toBool(true);
-                    s.supportsMzn = sj["supportsMzn"].toBool(false);
-                    if (sj["requiredFlags"].isArray()) {
-                        QJsonArray rfs = sj["requiredFlags"].toArray();
-                        for (auto rf : rfs) {
-                            s.requiredFlags.push_back(rf.toString());
-                        }
-                    }
-                    if (sj["stdFlags"].isArray()) {
-                        QJsonArray sfs = sj["stdFlags"].toArray();
-                        for (auto sf : sfs) {
-                            s.stdFlags.push_back(sf.toString());
-                        }
-                    }
-                    if (sj["extraFlags"].isArray()) {
-                        QJsonArray sfs = sj["extraFlags"].toArray();
-                        for (auto sf : sfs) {
-                            if (sf.isArray()) {
-                                QJsonArray extraFlagA = sf.toArray();
-                                if (extraFlagA.size()==4) {
-                                    SolverFlag extraFlag;
-                                    extraFlag.min=1.0;
-                                    extraFlag.max=0.0;
-                                    extraFlag.name = extraFlagA[0].toString();
-                                    QRegularExpression re_opt("^(int|float|bool)(:([0-9a-zA-Z.-]+):([0-9a-zA-Z.-]+))?");
-                                    QRegularExpressionMatch re_opt_match = re_opt.match(extraFlagA[2].toString());
-                                    if (re_opt_match.hasMatch()) {
-                                        if (re_opt_match.captured(1)=="int") {
-                                            if (re_opt_match.captured(3).isEmpty()) {
-                                                extraFlag.t = SolverFlag::T_INT;
-                                            } else {
-                                                extraFlag.t = SolverFlag::T_INT_RANGE;
-                                                extraFlag.min = re_opt_match.captured(3).toInt();
-                                                extraFlag.max = re_opt_match.captured(4).toInt();
-                                            }
-                                        } else if (re_opt_match.captured(1)=="float") {
-                                            if (re_opt_match.captured(3).isEmpty()) {
-                                                extraFlag.t = SolverFlag::T_FLOAT;
-                                            } else {
-                                                extraFlag.t = SolverFlag::T_FLOAT_RANGE;
-                                                extraFlag.min = re_opt_match.captured(3).toDouble();
-                                                extraFlag.max = re_opt_match.captured(4).toDouble();
-                                            }
-                                        } else if (re_opt_match.captured(1)=="bool") {
-                                            if (re_opt_match.captured(3).isEmpty()) {
-                                                extraFlag.t = SolverFlag::T_BOOL;
-                                            } else {
-                                                extraFlag.t = SolverFlag::T_BOOL_ONOFF;
-                                                extraFlag.options = QStringList({re_opt_match.captured(3),re_opt_match.captured(4)});
-                                            }
-                                        }
-                                    } else {
-                                        if (extraFlagA[2].toString()=="string") {
-                                            extraFlag.t = SolverFlag::T_STRING;
-                                        } else if (extraFlagA[2].toString().startsWith("opt:")) {
-                                            extraFlag.t = SolverFlag::T_OPT;
-                                            extraFlag.options = extraFlagA[2].toString().mid(4).split(":");
-//                                        } else if (extraFlagA[2].toString()=="solver") {
-//                                            extraFlag.t = SolverFlag::T_SOLVER;
-                                        } else {
-                                            continue;
-                                        }
-                                    }
-                                    extraFlag.description = extraFlagA[1].toString();
-                                    extraFlag.def = extraFlagA[3].toString();
-                                    s.extraFlags.push_back(extraFlag);
-                                }
-                            }
-                        }
-                    }
-                    s.isGUIApplication = sj["isGUIApplication"].toBool(false);
-                    s.needsMznExecutable = sj["needsMznExecutable"].toBool(false);
-                    s.needsStdlibDir = sj["needsStdlibDir"].toBool(false);
-                    s.needsPathsFile = sj["needsPathsFile"].toBool(false);
-                    s.needsSolns2Out = sj["needsSolns2Out"].toBool(true);
-                    s.executable = sj["executable"].toString("");
-                    s.id = sj["id"].toString();
-                    s.version = sj["version"].toString();
-                    s.mznlib = sj["mznlib"].toString();
-                    s.mznLibVersion = sj["mznlibVersion"].toInt();
-                    solvers.append(s);
-                }
-            }
-        }
-    }
-}
-
 void SolverDialog::editingFinished(bool showError)
 {
-    QString minizinc_executable;
-    QString minizinc_version;
+    auto& driver = MznDriver::get();
+    auto& solvers = driver.solvers();
     for (int i=0; i<solvers.size(); i++) {
         ui->solvers_combo->removeItem(0);
     }
-    solvers.clear();
-    checkMznExecutable(ui->mznDistribPath->text(),minizinc_executable,minizinc_version,solvers,userSolverConfigDir,userConfigFile,mznStdlibDir);
-    if (minizinc_executable.isEmpty()) {
-        if (showError) {
-            QMessageBox::warning(this,"MiniZinc IDE","Could not find the minizinc executable.",
-                                 QMessageBox::Ok);
-        }
-        if (minizinc_version.isEmpty()) {
-            ui->mzn2fzn_version->setText("None");
-        } else {
-            ui->mzn2fzn_version->setText("None. Error message:\n"+minizinc_version);
-        }
-    } else {
-        ui->mzn2fzn_version->setText(minizinc_version);
+    try {
+        driver.setLocation(ui->mznDistribPath->text());
+        ui->mzn2fzn_version->setText(driver.minizincVersionString());
         for (int i=solvers.size(); i--;) {
             ui->solvers_combo->insertItem(0,solvers[i].name+" "+solvers[i].version,i);
         }
         ui->solvers_combo->setCurrentIndex(0);
+    } catch (Exception& e) {
+        if (showError) {
+            QMessageBox::warning(this, "MiniZinc IDE", e.message(), QMessageBox::Ok);
+        }
+        ui->mzn2fzn_version->setText(e.message());
     }
 }
 
@@ -611,70 +625,6 @@ void SolverDialog::on_mznDistribPath_returnPressed()
     editingFinished(true);
 }
 
-void MznProcess::start(const QString &program, const QStringList &arguments, const QString &path)
-{
-    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-    QString curPath = env.value("PATH");
-    QString addPath = IDE::instance()->appDir();
-    if (!path.isEmpty())
-        addPath = path + pathSep + addPath;
-    env.insert("PATH", addPath + pathSep + curPath);
-    setProcessEnvironment(env);
-#ifdef Q_OS_WIN
-    _putenv_s("PATH", (addPath + pathSep + curPath).toStdString().c_str());
-    jobObject = CreateJobObject(nullptr, nullptr);
-    connect(this, SIGNAL(started()), this, SLOT(attachJob()));
-#else
-    setenv("PATH", (addPath + pathSep + curPath).toStdString().c_str(), 1);
-#endif
-    QProcess::start(program,arguments, QIODevice::Unbuffered | QIODevice::ReadWrite);
-#ifdef Q_OS_WIN
-    _putenv_s("PATH", curPath.toStdString().c_str());
-#else
-    setenv("PATH", curPath.toStdString().c_str(), 1);
-#endif
-}
-
-void MznProcess::terminate()
-{
-#ifdef Q_OS_WIN
-        AttachConsole(pid()->dwProcessId);
-        SetConsoleCtrlHandler(nullptr, TRUE);
-        GenerateConsoleCtrlEvent(CTRL_C_EVENT, 0);
-#else
-        ::killpg(processId(), SIGINT);
-#endif
-        if (!waitForFinished(500)) {
-            if (state() != QProcess::NotRunning) {
-#ifdef Q_OS_WIN
-                TerminateJobObject(jobObject, EXIT_FAILURE);
-#else
-                ::killpg(processId(), SIGKILL);
-#endif
-                if (!waitForFinished(500)) {
-                    kill();
-                    waitForFinished();
-                }
-            }
-        }
-}
-
-void MznProcess::setupChildProcess()
-{
-#ifndef Q_OS_WIN
-    if (::setpgid(0,0)) {
-        std::cerr << "Error: Failed to create sub-process\n";
-    }
-#endif
-}
-
-#ifdef Q_OS_WIN
-void MznProcess::attachJob()
-{
-    AssignProcessToJobObject(jobObject, pid()->hProcess);
-}
-#endif
-
 void SolverDialog::on_check_solver_clicked()
 {
     editingFinished(true);
@@ -692,3 +642,37 @@ void SolverDialog::on_mznlib_select_clicked()
     }
 
 }
+
+void SolverDialog::on_checkSolutions_checkBox_stateChanged(int checkState)
+{
+    QSettings settings;
+    settings.beginGroup("ide");
+    settings.setValue("checkSolutions", checkState == Qt::Checked);
+    settings.endGroup();
+}
+
+void SolverDialog::on_clearOutput_checkBox_stateChanged(int checkState)
+{
+    QSettings settings;
+    settings.beginGroup("ide");
+    settings.setValue("clearOutput", checkState == Qt::Checked);
+    settings.endGroup();
+}
+
+void SolverDialog::on_compressSolutions_checkBox_stateChanged(int checkState)
+{
+    bool checked = checkState == Qt::Checked;
+    ui->compressSolutions_spinBox->setEnabled(checked);
+    QSettings settings;
+    settings.beginGroup("ide");
+    settings.setValue("compressSolutions", checked ? ui->compressSolutions_spinBox->value() : 0);
+    settings.endGroup();
+}
+
+void SolverDialog::on_compressSolutions_spinBox_valueChanged(int value)
+{
+    QSettings settings;
+    settings.beginGroup("ide");
+    settings.setValue("compressSolutions", value);
+    settings.endGroup();
+}
